/*

*/

import java.util.Arrays;

/**
   A simple generic implementation of the Stack abstract data type
   that uses an array based implementation.
   <p>
   Here is the Stack interface.
      int     size()              <br>
      E       peek()              <br>
      E       pop()               <br>
      void    push(E element)     <br>
      boolean isEmpty()           <br>
*/
public class ArrayStack<E> extends Stack<E>
{
   private E[] data;
   private int itemCount;

   /**
      Default constructor creates an empty stack with a backing array of size 16.
   */
   public ArrayStack()
   {
      @SuppressWarnings("unchecked")
      final E[] data = (E[])new Object[16];
      this.data = data;

      itemCount = 0;
   }


   /**
      Returns the number of items currently in this {@code Stack}.

      @return the number of items in this {@code Stack}
   */
   @Override
   public int size()
   {
      return itemCount;
   }


   /**
      Return the item at the top of this {@code Stack}.
      <p>
      The top of the stack is at the end of the array.

      @return a reference to the item at the top of this {@code Stack}
   */
   @Override
   public E peek()
   {
      return data[itemCount - 1];
   }


   /**
      Remove the item at the top of this {@code Stack}.
      The {@code Stack} size decreases by one.
      <p>
      The top of the stack is at the end of the array.

      @return a reference to the item that was removed from the top of this {@code Stack}
   */
   @Override
   public E pop()
   {
      E temp = data[itemCount - 1];
      --itemCount;
      return temp;
   }


   /**
      Insert the item at the top of this {@code Stack}.
      The {@code Stack} size increases by one.
      <p>
      The top of the stack is at the end of the array.

      @param element  reference to the item to be added to the top of this {@code Stack}
   */
   @Override
   public void push(E element)
   {
      if (itemCount == data.length)
      {
         data = Arrays.copyOf( data,    2*data.length ); // good for small arrays
       //data = Arrays.copyOf( data, 1000+data.length ); // good for large arrays
       //data = Arrays.copyOf( data,    1+data.length ); // bad idea
      }

      this.data[itemCount] = element;
      ++itemCount;
   }


   /**
      Return a {@link String} representation of this {@code Stack}.

      @return {@link String} representation of this {@code Stack}
   */
   @Override
   public String toString()
   {
      String result = "<";
      for (int i = 0; i < itemCount; ++i)
      {
         result += data[i];
         if (i < itemCount - 1)
            result += ", ";
      }
      result += ">";
      return result;
   }

}//ArrayStack
