/*

*/



/**
   A simple generic implementation of the Stack abstract data type
   that uses a linked list based implementation.
   <p>
   Here is the Stack interface.
      int     size()              <br>
      E       peek()              <br>
      E       pop()               <br>
      void    push(E element)     <br>
      boolean isEmpty()           <br>
*/
public class LinkedStack<E> extends Stack<E>
{
   private Node<E> top;
   private int itemCount;

   /**
      Default constructor creates an empty stack.
   */
   public LinkedStack()
   {
      top = null;
      itemCount = 0;
   }


   /**
      Returns the number of items currently in this {@code Stack}.

      @return the number of items in this {@code Stack}
   */
   @Override
   public int size()
   {
      return itemCount;
   }


   /**
      Return the item at the top of this {@code Stack}.

      @return a reference to the item at the top of this {@code Stack}
   */
   @Override
   public E peek()
   {
      return top.getData();
   }


   /**
      Remove the item at the top of this {@code Stack}.
      The {@code Stack} size decreases by one.

      @return a reference to the item that was removed from the top of this {@code Stack}
   */
   @Override
   public E pop()
   {
      E item = top.getData();
      top = top.getNext();
      --itemCount;
      return item;
   }


   /**
      Insert the item at the top of this {@code Stack}.
      The {@code Stack} size increases by one.

      @param element  reference to the item to be added to the top of this {@code Stack}
   */
   @Override
   public void push(E element)
   {
      Node<E> node = new Node<>(element, top);
      top = node;
      ++itemCount;
   }


   /**
      Return a {@link String} representation of this {@code Stack}.

      @return {@link String} representation of this {@code Stack}
   */
   @Override
   public String toString()
   {
      String result = "<";
      Node<E> cursor = top;
      if (null != cursor) // handle the first node
      {
         result += cursor.getData();
         cursor = cursor.getNext();
      }
      while (null != cursor) // start the loop on the second node
      {
         result += ", " + cursor.getData();
         cursor = cursor.getNext();
      }

      result += ">";
      return result;
   }

}//LinkedStack



/**
   This is a simple, private, generic Node class.
*/
class Node<E>
{
   private E data;
   private Node<E> next;

   public Node(E data, Node<E> next)
   {
      this.data = data;
      this.next = next;
   }

   public E       getData( )            {return data;}
   public Node<E> getNext( )            {return next;}
   public void    setData(E data)       {this.data = data;}
   public void    setNext(Node<E> next) {this.next = next;}
}
