/*

*/

import java.util.Arrays;

/**
   A simple generic implementation of the Stack abstract data type
   that uses an ArrayList based implementation.
   <p>
   Here is the Stack interface.
      int     size()              <br>
      E       peek()              <br>
      E       pop()               <br>
      void    push(E element)     <br>
      boolean isEmpty()           <br>
*/
public class ArrayStack<E> extends Stack<E>
{
   private List<E> list;

   /**
      Default constructor creates an empty stack.
   */
   public ArrayStack()
   {
      list = new ArrayList<>();
   }


   /**
      Returns the number of items currently in this {@code Stack}.

      @return the number of items in this {@code Stack}
   */
   @Override
   public int size()
   {
      return list.size();
   }


   /**
      Return the item at the top of this {@code Stack}.
      <p>
      The top of the stack is at the end of the list.

      @return a reference to the item at the top of this {@code Stack}
   */
   @Override
   public E peek()
   {
      return list.get(list.size() - 1);
   }


   /**
      Remove the item at the top of this {@code Stack}.
      The {@code Stack} size decreases by one.
      <p>
      The top of the stack is at the end of the list.

      @return a reference to the item that was removed from the top of this {@code Stack}
   */
   @Override
   public E pop()
   {
      return list.remove(list.size() - 1);
   }


   /**
      Insert the item at the top of this {@code Stack}.
      The {@code Stack} size increases by one.
      <p>
      The top of the stack is at the end of the list.

      @param element  reference to the item to be added to the top of this {@code Stack}
   */
   @Override
   public void push(E element)
   {
      list.add(list.size(), element);
   }


   /**
      Return a {@link String} representation of this {@code Stack}.

      @return {@link String} representation of this {@code Stack}
   */
   @Override
   public String toString()
   {
      return list.toString();
   }

}//ArrayStack
