/*

*/

import java.util.Arrays;

/**
   A simple generic implementation of the Stack abstract data type
   that uses an ArrayList implementation.
   <p>
   Here is the Stack interface.
      E     peek()              <br>
      E     pop()               <br>
      void  push(E element)     <br>
*/
public class ArrayStack<E> extends ArrayList<E> implements Stack<E>
{
   private List<E> stack;

   /**
      Default constructor creates an empty stack.
   */
   public ArrayStack()
   {
      super();
   }


   /**
      Return the item at the top of this {@code Stack}.
      <p>
      The top of the stack is at the end of the list.

      @return a reference to the item at the top of this {@code Stack}
   */
   @Override
   public E peek()
   {
      return this.get(this.size() - 1);
   }


   /**
      Remove the item at the top of this {@code Stack}.
      The {@code Stack} size decreases by one.
      <p>
      The top of the stack is at the end of the list.

      @return a reference to the item that was removed from the top of this {@code Stack}
   */
   @Override
   public E pop()
   {
      return this.remove(this.size() - 1);
   }


   /**
      Insert the item at the top of this {@code Stack}.
      The {@code Stack} size increases by one.
      <p>
      The top of the stack is at the end of the list.

      @param element  reference to the item to be added to the top of this {@code Stack}
   */
   @Override
   public void push(E element)
   {
      this.add(this.size(), element);
   }

}//ArrayStack
